# AUTOGENERATED! DO NOT EDIT! File to edit: ../10_xbee_coding.ipynb.

# %% auto 0
__all__ = ['np_array_to_Odometry', 'xbee_encode', 'xbee_decode']

# %% ../10_xbee_coding.ipynb 4
import pickle

def np_array_to_Odometry(array):
    msg = Odometry()
    msg.header.frame_id = "odom"
    msg.pose.pose.position.x = array[0]
    msg.pose.pose.position.y = array[1]
    msg.pose.pose.position.z = array[2]
    msg.pose.pose.orientation.x = array[3]
    msg.pose.pose.orientation.y = array[4]
    msg.pose.pose.orientation.z = array[5]
    msg.pose.pose.orientation.w = array[6]
    msg.twist.twist.linear.x = array[7]
    msg.twist.twist.linear.y = array[8]
    msg.twist.twist.linear.z = array[9]
    msg.twist.twist.angular.x = array[10]
    msg.twist.twist.angular.y = array[11]
    msg.twist.twist.angular.z = array[12]
    return msg

def xbee_encode(data_via_xbee, data_type):
    data = data_via_xbee

    # send data
    byte_arr = pickle.dumps( data )
    length, index, check= int(len(byte_arr)), 0, 0

    for index in range(0,length,250) :
        pack = bytearray(b'\xAB') #Header
        pack.extend(bytearray(data_type)) #Type
        pack.extend( length.to_bytes(4, byteorder='big') ) #bytes
        index_end = index+250 if index+250 < length else length
        pack.extend( byte_arr[index:(index_end)] ) #data

        if index_end == length : pack.extend(check.to_bytes(1, byteorder='big')) # checksum
        else: check = 0xff & (check + pack[-1])

    return pack  

def xbee_decode(xbee_message):
    print(xbee_message)

    if not xbee_message[0:1] == b'\xAB' : # Header wrong
        print('get xbee_message with wrong Header')
        return

    if not ((xbee_message[1:2] == b'\x00') or (xbee_message[1:2] == b'\x01') or (xbee_message[1:2] == b'\x02') or (xbee_message[1:2] == b'\x03')):
        rospy.loginfo('xbeejoy callback')
        self.count += 1
        get_msg = pickle.loads(xbee_message)
        axes = get_msg[0:8]
        buttons = get_msg[8:]
        rospy.loginfo(axes)
        rospy.loginfo(buttons)

        msg = Joy()
        msg.header.seq = self.count
        msg.header.frame_id = "/dev/input/js0"
        msg.header.stamp = rospy.Time.now()
        msg.axes = axes
        msg.buttons = buttons
        return msg
    
    get_register.extend(xbee_message[6:])

    if xbee_message[1:2] == b'\x00':
        get_msg = pickle.loads(get_register[:-1])
        print(get_msg)
        return(get_msg)
    
    
    if xbee_message[1:2] == b'\x02': # type: points
        get_points = pickle.loads(get_register[:-1])
        pub_msg = np_array_to_Odometry(get_points)
        return(pub_msg)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../06_uwb.ipynb.

# %% auto 0
__all__ = ['UWB']

# %% ../06_uwb.ipynb 4
import yaml
import pypozyx
from pypozyx import NetworkID, Coordinates, PozyxSerial, DeviceCoordinates, DeviceRange, PozyxConstants
from pypozyx.core import PozyxException

# %% ../06_uwb.ipynb 5
class UWB():
    def __init__(self, port = None):
        self.port = port
        self._network_id = None
        self._pozyx_handler = None        
        self._pose = None
        self._env_config = None
        
        #TODO: Make height parameterized
        self._height = 500
        
    @property
    def network_id(self):
        return self._network_id
    
    @network_id.setter
    def network_id(self, value=None):
        if value is None:
            self._network_id = NetworkID()
        else:
            self._network_id = NetworkID(value)
        
    @property
    def port(self):
        return self._port
    
    @port.setter
    def port(self, port):
        self._port = port
        
    @property
    def pose(self):
        return self._pose
    
    @pose.setter
    def pose(self, value=None):
        if value is None:
            self._pose = Coordinates()
        else:
            self._pose.x = value[0]
            self._pose.y = value[1]
            self._pose.z = value[2]
    
    @property
    def env_config(self):
        return self._env_config
    
    def load_env_config(self, config_file_path: str) -> bool:
        """
        Load UWB anchors' environment config.
        
        Args:
            config_file_path (str): The environment config file path.
            
        Returns:
            bool: True for success, False for failure.
        """
        with open(config_file_path, 'r') as config_file:
            try:
                self._env_config = yaml.safe_load(config_file)
            except yaml.YAMLError as ex:
                print(ex)
                return False
        return True
    
    def connect(self):
        if self.port is None:
            #TODO:
            pass
        else:
            try:
                self.pozyx_handler = PozyxSerial(self.port)
            except PozyxException:
                #TODO:
                pass
    
    def write_device(self):
        ANCHOR_FLAG = 1
        self.pozyx_handler.clearDevices()
        for anchor_name, config in self.env_config:
            coordinate = Coordinates(config['x'], config['y'], config['z'])
            device_coordinate = DeviceCoordinates(config['id'], ANCHOR_FLAG, coordinate)
            self.pozyx_handler.addDevice(device_coordinate)
            self.pozyx_handler.setSelectionOfAnchorsAutomatic(len(self.env_config))
    
    #TODO: Seperate localize of differ dimensions and height
    def localize(self, dest_id):
        self.pozyx_handler.doPosition(self.pose, PozyxConstants.DIMENSION_3D, self._height, PozyxConstants.POSITIONING_ALGORITHM_UWB_ONLY, )
        
    def range(self, dest_id):
        ranges = DeviceRange()
        self.pozyx_handler.doRanging(dest_id, ranges)
        return ranges



# Autogenerated by nbdev

d = { 'settings': { 'audience': 'Developers',
                'author': 'hchengwang',
                'author_email': 'hchengwang@gmail.com',
                'branch': 'master',
                'company_name': 'ARG @ NYCU',
                'copyright': 'ARG @ NYCU',
                'custom_sidebar': 'False',
                'description': 'A toolset for navigation (no ros) created by arg lab',
                'doc_baseurl': '/arg_robotics_tools/',
                'doc_host': 'https://ARG-NCTU.github.io',
                'doc_path': 'docs',
                'git_url': 'https://github.com/ARG-NCTU/arg_robotics_tools/tree/master/',
                'host': 'github',
                'keywords': 'navi tools utils',
                'language': 'English',
                'lib_name': 'arg_robotics_tools',
                'lib_path': 'arg_robotics_tools',
                'license': 'apache2',
                'min_python': '3.6',
                'nbs_path': '.',
                'recursive': 'False',
                'repo_name': 'arg_robotics_tools',
                'status': '2',
                'title': 'arg_robotics_tools',
                'user': 'ARG-NCTU',
                'version': '0.0.1'},
  'syms': { 'arg_robotics_tools.anchor_logging': { 'arg_robotics_tools.anchor_logging.examine_one_plot': 'https://ARG-NCTU.github.io/arg_robotics_tools/anchor_logging.html#examine_one_plot',
                                                   'arg_robotics_tools.anchor_logging.examine_one_with_boat_plot': 'https://ARG-NCTU.github.io/arg_robotics_tools/anchor_logging.html#examine_one_with_boat_plot',
                                                   'arg_robotics_tools.anchor_logging.examine_plot': 'https://ARG-NCTU.github.io/arg_robotics_tools/anchor_logging.html#examine_plot'},
            'arg_robotics_tools.camera_projection': { 'arg_robotics_tools.camera_projection.camera_projection': 'https://ARG-NCTU.github.io/arg_robotics_tools/camera_projection.html#camera_projection',
                                                      'arg_robotics_tools.camera_projection.camera_projection.apriltag_detection': 'https://ARG-NCTU.github.io/arg_robotics_tools/camera_projection.html#camera_projection.apriltag_detection',
                                                      'arg_robotics_tools.camera_projection.camera_projection.draw_point': 'https://ARG-NCTU.github.io/arg_robotics_tools/camera_projection.html#camera_projection.draw_point',
                                                      'arg_robotics_tools.camera_projection.camera_projection.read_camera_info': 'https://ARG-NCTU.github.io/arg_robotics_tools/camera_projection.html#camera_projection.read_camera_info',
                                                      'arg_robotics_tools.camera_projection.camera_projection.read_images': 'https://ARG-NCTU.github.io/arg_robotics_tools/camera_projection.html#camera_projection.read_images',
                                                      'arg_robotics_tools.camera_projection.camera_projection.solvePnP': 'https://ARG-NCTU.github.io/arg_robotics_tools/camera_projection.html#camera_projection.solvepnp'},
            'arg_robotics_tools.get_ip': { 'arg_robotics_tools.get_ip.device_get_sensors': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#device_get_sensors',
                                           'arg_robotics_tools.get_ip.dp_get_devices': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#dp_get_devices',
                                           'arg_robotics_tools.get_ip.dp_load_config': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#dp_load_config',
                                           'arg_robotics_tools.get_ip.find_duckiepond_devices_yaml': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#find_duckiepond_devices_yaml',
                                           'arg_robotics_tools.get_ip.get_key': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#get_key',
                                           'arg_robotics_tools.get_ip.get_xbee_address': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#get_xbee_address',
                                           'arg_robotics_tools.get_ip.get_xbee_address_boat': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#get_xbee_address_boat',
                                           'arg_robotics_tools.get_ip.ip_connection': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#ip_connection',
                                           'arg_robotics_tools.get_ip.myip': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#myip',
                                           'arg_robotics_tools.get_ip.sensor_get_topic': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#sensor_get_topic',
                                           'arg_robotics_tools.get_ip.ssh_connection': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#ssh_connection',
                                           'arg_robotics_tools.get_ip.ssh_ping_nano': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#ssh_ping_nano',
                                           'arg_robotics_tools.get_ip.ssh_ping_rpi': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#ssh_ping_rpi',
                                           'arg_robotics_tools.get_ip.ssh_rostopic': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#ssh_rostopic',
                                           'arg_robotics_tools.get_ip.test_ping': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#test_ping',
                                           'arg_robotics_tools.get_ip.test_rostopic': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#test_rostopic',
                                           'arg_robotics_tools.get_ip.test_ssh': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#test_ssh',
                                           'arg_robotics_tools.get_ip.test_ssh_intranet': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#test_ssh_intranet',
                                           'arg_robotics_tools.get_ip.whoami': 'https://ARG-NCTU.github.io/arg_robotics_tools/get_ip.html#whoami'},
            'arg_robotics_tools.random_map': { 'arg_robotics_tools.random_map.draw_line': 'https://ARG-NCTU.github.io/arg_robotics_tools/random_map.html#draw_line',
                                               'arg_robotics_tools.random_map.random_generate': 'https://ARG-NCTU.github.io/arg_robotics_tools/random_map.html#random_generate',
                                               'arg_robotics_tools.random_map.sub_random_generate': 'https://ARG-NCTU.github.io/arg_robotics_tools/random_map.html#sub_random_generate'},
            'arg_robotics_tools.robot_model': { 'arg_robotics_tools.robot_model.robot_model_loader': 'https://ARG-NCTU.github.io/arg_robotics_tools/robot_model.html#robot_model_loader',
                                                'arg_robotics_tools.robot_model.robot_model_loader.list_all': 'https://ARG-NCTU.github.io/arg_robotics_tools/robot_model.html#robot_model_loader.list_all',
                                                'arg_robotics_tools.robot_model.robot_model_loader.load': 'https://ARG-NCTU.github.io/arg_robotics_tools/robot_model.html#robot_model_loader.load',
                                                'arg_robotics_tools.robot_model.robot_model_loader.show_link': 'https://ARG-NCTU.github.io/arg_robotics_tools/robot_model.html#robot_model_loader.show_link'},
            'arg_robotics_tools.tsp': { 'arg_robotics_tools.tsp.dist': 'https://ARG-NCTU.github.io/arg_robotics_tools/tsp.html#dist',
                                        'arg_robotics_tools.tsp.distanceGenerate': 'https://ARG-NCTU.github.io/arg_robotics_tools/tsp.html#distancegenerate',
                                        'arg_robotics_tools.tsp.solve_tsp_held_karp': 'https://ARG-NCTU.github.io/arg_robotics_tools/tsp.html#solve_tsp_held_karp',
                                        'arg_robotics_tools.tsp.solve_tsp_nearest_neighbor': 'https://ARG-NCTU.github.io/arg_robotics_tools/tsp.html#solve_tsp_nearest_neighbor',
                                        'arg_robotics_tools.tsp.sortWaypoint': 'https://ARG-NCTU.github.io/arg_robotics_tools/tsp.html#sortwaypoint'},
            'arg_robotics_tools.utils': { 'arg_robotics_tools.utils.gdown_unzip': 'https://ARG-NCTU.github.io/arg_robotics_tools/utils.html#gdown_unzip',
                                          'arg_robotics_tools.utils.pose_dis': 'https://ARG-NCTU.github.io/arg_robotics_tools/utils.html#pose_dis',
                                          'arg_robotics_tools.utils.waypoint': 'https://ARG-NCTU.github.io/arg_robotics_tools/utils.html#waypoint'},
            'arg_robotics_tools.uwb': { 'arg_robotics_tools.uwb.UWB': 'https://ARG-NCTU.github.io/arg_robotics_tools/uwb.html#uwb',
                                        'arg_robotics_tools.uwb.UWB.connect': 'https://ARG-NCTU.github.io/arg_robotics_tools/uwb.html#uwb.connect',
                                        'arg_robotics_tools.uwb.UWB.env_config': 'https://ARG-NCTU.github.io/arg_robotics_tools/uwb.html#uwb.env_config',
                                        'arg_robotics_tools.uwb.UWB.load_env_config': 'https://ARG-NCTU.github.io/arg_robotics_tools/uwb.html#uwb.load_env_config',
                                        'arg_robotics_tools.uwb.UWB.localize': 'https://ARG-NCTU.github.io/arg_robotics_tools/uwb.html#uwb.localize',
                                        'arg_robotics_tools.uwb.UWB.network_id': 'https://ARG-NCTU.github.io/arg_robotics_tools/uwb.html#uwb.network_id',
                                        'arg_robotics_tools.uwb.UWB.port': 'https://ARG-NCTU.github.io/arg_robotics_tools/uwb.html#uwb.port',
                                        'arg_robotics_tools.uwb.UWB.pose': 'https://ARG-NCTU.github.io/arg_robotics_tools/uwb.html#uwb.pose',
                                        'arg_robotics_tools.uwb.UWB.range': 'https://ARG-NCTU.github.io/arg_robotics_tools/uwb.html#uwb.range',
                                        'arg_robotics_tools.uwb.UWB.write_device': 'https://ARG-NCTU.github.io/arg_robotics_tools/uwb.html#uwb.write_device'},
            'arg_robotics_tools.video2picture': { 'arg_robotics_tools.video2picture.get_images_from_video': 'https://ARG-NCTU.github.io/arg_robotics_tools/video2picture.html#get_images_from_video'},
            'arg_robotics_tools.websocket_rosbridge': { 'arg_robotics_tools.websocket_rosbridge.ros_socket': 'https://ARG-NCTU.github.io/arg_robotics_tools/websocket_rosbridge.html#ros_socket',
                                                        'arg_robotics_tools.websocket_rosbridge.ros_socket.check_connecting': 'https://ARG-NCTU.github.io/arg_robotics_tools/websocket_rosbridge.html#ros_socket.check_connecting',
                                                        'arg_robotics_tools.websocket_rosbridge.ros_socket.get_node': 'https://ARG-NCTU.github.io/arg_robotics_tools/websocket_rosbridge.html#ros_socket.get_node',
                                                        'arg_robotics_tools.websocket_rosbridge.ros_socket.get_topic': 'https://ARG-NCTU.github.io/arg_robotics_tools/websocket_rosbridge.html#ros_socket.get_topic',
                                                        'arg_robotics_tools.websocket_rosbridge.ros_socket.println': 'https://ARG-NCTU.github.io/arg_robotics_tools/websocket_rosbridge.html#ros_socket.println',
                                                        'arg_robotics_tools.websocket_rosbridge.ros_socket.publisher': 'https://ARG-NCTU.github.io/arg_robotics_tools/websocket_rosbridge.html#ros_socket.publisher',
                                                        'arg_robotics_tools.websocket_rosbridge.ros_socket.subscriber': 'https://ARG-NCTU.github.io/arg_robotics_tools/websocket_rosbridge.html#ros_socket.subscriber'},
            'arg_robotics_tools.xbee_coding': { 'arg_robotics_tools.xbee_coding.np_array_to_Odometry': 'https://ARG-NCTU.github.io/arg_robotics_tools/xbee_coding.html#np_array_to_odometry',
                                                'arg_robotics_tools.xbee_coding.xbee_decode': 'https://ARG-NCTU.github.io/arg_robotics_tools/xbee_coding.html#xbee_decode',
                                                'arg_robotics_tools.xbee_coding.xbee_encode': 'https://ARG-NCTU.github.io/arg_robotics_tools/xbee_coding.html#xbee_encode'}}}


# AUTOGENERATED! DO NOT EDIT! File to edit: ../09_anchor_logging.ipynb.

# %% auto 0
__all__ = ['examine_plot', 'examine_one_plot', 'examine_one_with_boat_plot']

# %% ../09_anchor_logging.ipynb 4
from . import utils
import matplotlib.pyplot as plt
import numpy as np

# %% ../09_anchor_logging.ipynb 5
def examine_plot(lines):
    plt.rcParams["figure.figsize"] = [8.00, 5.00]
    plt.rcParams["figure.autolayout"] = True
    fig = plt.figure()
    ax = fig.add_subplot(111)
    value = []
    for line in lines:
        if (line.find("time") == -1):#is not time line
            temp_list = line.split()
            while 'boat_alive' in temp_list: 
                temp_list.remove('boat_alive')
            value.append(float(len(temp_list)))
    x_axis = np.array([i for i in range(int(count/2)+1)])
    ax.plot(x_axis, value)
    hours = int(count/2/1800)+2
    hour_points=[]
    for i in range(hours):
        hour_points.append(float(1800*i))
    grid_points = hour_points
    ax.xaxis.set_ticks(grid_points)
    ax.grid(True)
    plt.xlabel("Time")
    plt.ylabel("Anchors")
    plt.show()

# %% ../09_anchor_logging.ipynb 6
def examine_one_plot(anchor, lines):
    plt.rcParams["figure.figsize"] = [5.00, 3.00]
    value = []
    for line in lines:
        if (line.find("time") == -1):#is not time line
            temp_list = line.split()
            if anchor in temp_list: 
                value.append(float(1))
            else:
                value.append(float(0))
    x_axis = np.array([i for i in range(int(count/2)+1)])
    plt.grid(visible=True, axis='x')
    plt.plot(x_axis, value)
    plt.xlabel("Time")
    plt.ylabel(anchor)
    plt.show()

# %% ../09_anchor_logging.ipynb 7
def examine_one_with_boat_plot(anchor, lines):
    plt.rcParams["figure.figsize"] = [5.00, 3.00]
    value = []
    boat_value = []
    for line in lines:
        if (line.find("time") == -1):#is not time line
            temp_list = line.split()
            if anchor in temp_list: 
                value.append(float(1))
                if 'boat_alive'in temp_list:
                    if (temp_list.index(anchor)+2) == temp_list.index('boat_alive'):
                        boat_value.append(float(1))
                    else:
                        boat_value.append(float(0))
                else:
                    boat_value.append(float(0))
            else:
                value.append(float(0))
                boat_value.append(float(0))
    x_axis = np.array([i for i in range(int(count/2)+1)])
    plt.grid(visible=True, axis='x')
    plt.plot(x_axis, value, x_axis, boat_value)
    plt.xlabel("Time")
    plt.ylabel(anchor)
    plt.show()
